# frozen_string_literal: true
require 'ostruct'
module Steamd
  module Generator
    # Represents an class variable from the Generator
    class Variable
      # The name of the Variable
      attr_reader :name

      # The variable Type
      attr_reader :type

      # The variable size
      attr_reader :size

      # The variable modifier
      attr_reader :modifier

      # The variable modifier size
      attr_reader :modifier_size

      # The variable value
      attr_reader :value

      # Create a new GeneratedClass
      def initialize(opts)
        @name = opts[:name]
        @value = opts[:value]
        @type = opts[:type]
        @modifier = opts[:modifier]
        @modifier_size = opts[:modifier_size]
      end

      # The name of the Variable
      #
      # @return [String] variable name, underscored
      def name
        @name.underscore
      end

      # The size of the variable modifier
      #
      # @return [String]
      def modifier_size
        if @modifier_size.is_a?(String)
          @modifier_size.underscore
        else
          @modifier_size
        end
      end

      # The type of the variable
      def type
        case @type
        when 'SteamKit2.Internal.CMsgProtoBufHeader'
          'Steamclient::CMsgProtoBufHeader'
        when 'SteamKit2.GC.Internal.CMsgProtoBufHeader'
          'Gc::CMsgProtoBufHeader'
        else
          @type
        end
      end

      def value
        if @value.is_a?(String)
          parts = @value.split('::')
          name = parts.last.underscore.upcase
          [parts.first, name].join('::')
        else
          @value
        end
      end

      # Return the variable as a hash
      def to_hash
        {
          name: name,
          type: type,
          modifier: modifier,
          value: value,
          size: size,
          modifier_size: modifier_size
        }
      end
      alias to_h to_hash
    end

    # Class constant
    class Constant < Variable
      attr_reader :name, :value
      def name
        @name.underscore.upcase
      end
    end

    # Represents an Class from the Generator
    class GeneratedClass
      # Create a Generated Class
      def initialize(klass)
        @klass = klass
      end

      # List of variables to be used in the class
      def variables
        @klass[:variables].select { |v| v[:modifier] != 'const' }.map do |var|
          Variable.new(var)
        end
      end

      # List of constants to be used in the class
      def constants
        @klass[:variables].select { |v| v[:modifier] == 'const' }.map do |const|
          Constant.new(const)
        end
      end

      # The class name
      def name
        @klass[:name]
      end

      # Binding for ERB
      def erb_binding
        binding
      end
    end
  end
end
