# frozen_string_literal: true
require 'erb'
require 'ostruct'

require 'steamd/generator/generated_class'
require 'steamd/generator/generated_enum'
require 'steamd/generator/generated_import'

module Steamd
  module Generator
    # The underlying Implementation of a generator.
    #
    # @example Making a generate
    #   class MyGenerator
    #     include Generator::Implementation
    #
    #     def generate
    #       classes.each { |c| puts c.name }
    #     end
    #   end
    module Implementation
      # Instantiate the object with a given IO stream.
      #
      # @param io The IO object
      def initialize(io)
        @parser = Steamd::Parser.new
        @buffer = ''
        @io = io
        self
      end

      # Run the generator. Load the parser, and parse the IO object.
      #
      # @return The parsed code
      def run
        @parser.load!
        @parser.parse(@io)
        generate
        @buffer
      end

      # The classes parsed from the lanaguage
      def classes
        @parser.classes.map do |klass|
          GeneratedClass.new(klass)
        end
      end

      # The imports parsed from the language
      def imports
        @parser.imports.map do |import|
          GeneratedImport.new(import)
        end
      end

      # The enums parsed from the language
      def enums
        @parser.enums.map do |enum|
          GeneratedEnum.new(enum)
        end
      end

      private

      # Generates the code
      def generate(_io)
        raise NotImplementedError
      end

      # @api private
      def erb(template, vars)
        ERB.new(template, nil, '-').result(vars.erb_binding)
      end

      # Append a string to the internal buffer
      #
      # @api private
      def append(str)
        @buffer += str
      end
    end
  end
end
