# frozen_string_literal: true
# Serializable type
class Type
  attr_reader :type

  # Instantiate a Type
  #
  # @example Instantiate a Type
  #   type = Type.new('int')
  def initialize(type)
    @type = type
  end

  # The String representation of the Type. If it is a SteamKit Internal type,
  # the type is transformed into namespaces from the `steam-proto` repo.
  #
  # @return [String] the type
  def type
    case @type
    when 'SteamKit2.Internal.CMsgProtoBufHeader'
      'Steamclient::CMsgProtoBufHeader'
    when 'SteamKit2.GC.Internal.CMsgProtoBufHeader'
      'Gc::CMsgProtoBufHeader'
    when *primitives
      @type
    else
      Object.const_get(@type.to_s).to_s
    end
  end

  # Is this type an encodable type?
  #
  # @return [Bool]
  def encodable?
    encodables.include?(type)
  end

  # Is this type an primitive type?
  #
  # @return [Bool]
  def primitive?
    primitives.include?(type)
  end

  # The byte directive for this type
  #
  # @return [String]
  def bytedirective
    {
      'int' => 'l<',
      'ushort' => 'S<',
      'short' => 's<',
      'byte' => 'c',
      'uint' => 'L<',
      'ulong' => 'Q<'
    }[type]
  end

  # The size in bytes of this type
  #
  # @return [Integer]
  def bytesize
    case type
    when 'uint', 'int'
      4
    when 'ushort', 'short'
      2
    when 'ulong', 'long'
      8
    when 'byte'
      1
    end
  end

  # The Class represented by this Type
  #
  # @return [Object]
  def klass
    type.constantize
  end

  private

  # @api private
  def primitives
    %w(int short uint long ulong byte ushort)
  end

  # @api private
  def encodables
    ['Steamclient::CMsgProtoBufHeader', 'Gc::CMsgProtoBufHeader']
  end
end
