# frozen_string_literal: true
# Base variable, each variable has a type and a name at the very least.
class Variable < Node
  # Returns the name of the variable
  #
  # @return [String] the variable name
  def name
    variable_name.text_value
  end

  # Returns the type of the variable
  #
  # @return [String] the variable type
  def type
    var_type.type.text_value
  end

  # Returns the size, if any of the variable. ie byte<6> y;
  #
  # @return [String/Integer/nil] the variable size
  def size
    return nil unless var_type && var_type.respond_to?(:size)

    if var_type.size.text_value.empty?
      nil
    else
      var_type.size.var_value.value
    end
  end

  # Hash representation of this Variable Node
  #
  # @return [Hash]
  def to_hash
    {
      name: name,
      type: type,
      modifier: nil,
      value: nil,
      size: size,
      modifier_size: nil
    }
  end
end

# Variable with a modifier.
#
# ie: proto int x;
class VariableWithModifier < Variable
  # Determins if the modifier of this variable is sized.
  #
  # ie: proto<len> CMsgHeaderProtobuf x;
  def sized?
    var_modifier.respond_to?(:size) &&
      !var_modifier.size.text_value.empty?
  end

  # The size of the variable modifier
  def modifier_size
    return nil if var_type.nil?

    var_modifier.size.value if sized?
  end
end

# A variable with a modifier but no value
# ie: proto header hi;
class VariableWithModifierNoValue < VariableWithModifier
  # Hash representation of the VariableWithModifierNoValue
  #
  # @return [Hash]
  def to_hash
    hsh = super

    hsh.merge!(
      modifier: var_modifier.word.text_value,
      modifier_size: modifier_size
    )
  end
end

# A variable with a modifier and a default value
class VariableWithModifierWithValue < VariableWithModifierNoValue
  # Hash representation of the VariableWithModifierWithValue
  #
  # @return [Hash]
  def to_hash
    hsh = super

    hsh.merge!(
      value: var_value.value
    )
  end
end

# Variable, no modifier with a value
# ie: int x = 1;
class VariableWithValue < Variable
  # Hash representation of the VariableWithValue
  #
  # @return [Hash]
  def to_hash
    hsh = super
    hsh.merge!(value: var_value.value)
  end
end
