# frozen_string_literal: true
require 'treetop'
require 'steamd/nodes/nodes'

module Steamd
  # Parses steam language files. Give access to the parsed data
  # via #classes, #imports and #enums methods.
  #
  # @example Parsing a Steam Language file
  #   parser = Parser.new
  #   parser.load! # loads the grammars, sets up the parser
  #   parser.parse(File.open('./message.steamd'))
  class Parser
    # The base Parser error
    Error = Class.new(StandardError)

    # Thrown if there was any non specific Parse error.
    ParseError = Class.new(Error)

    # Thrown if the the parse method is called before the parser is loaded
    NotLoadedError = Class.new(Error)

    # Thrown if the callee tries to access the parsed data before parse is
    # called
    StreamNotParsed = Class.new(Error)

    # Load the grammars
    #
    # @return [Bool] returns true unless an exception is thrown
    def load!
      return if loaded?

      Treetop.load("#{Steamd.grammar_dir}/shared.treetop")

      Dir.glob("#{Steamd.grammar_dir}/*.treetop") do |file|
        Treetop.load(file)
      end

      @parser = SteamdParser.new
      true
    end

    # Parses an IO stream.
    #
    # @param io [#read, #rewind] An IO stream containing the Steam Language
    #   data.
    # @return [Bool] returns true unless an error is thrown
    def parse(io)
      io.rewind
      raise NotLoadedError, 'load before parsing (#load!)' if @parser.nil?

      data = strip_comments_and_obsolete_tags!(io)

      @tree = @parser.parse(data)
      raise_parser_error! if @tree.nil?

      true
    end

    # An array of hashes containing the imports found in the parsed
    # Steam Language file.
    #
    # The hash is the hash version of ImportStatement
    #
    # @example Accessing imports
    #   parser.imports # => [{filename: 'test'}]
    #
    # @return [Array<Hash>] an array of hashes representing the imports found
    # @see ImportStatement
    def imports
      raise StreamNotParsed, 'you must parse first' if @tree.nil?

      imports = statements.select do |node|
        node.is_a?(ImportStatement)
      end

      imports.map(&:to_hash)
    end

    # Returns an array of classes parsed from the IO stream. The array
    # contains hashes with class information and variables.
    #
    # The hash is the hash form of ClassStatement
    #
    # @example Accessing the classes
    #   parser.classes # => [{ name: 'test', variables: [], ...]
    #
    # @return [Array<Hash>] an array of hashes representing the class
    # @see ClassStatement
    def classes
      raise StreamNotParsed, 'you must parse first' if @tree.nil?

      classes = statements.select do |node|
        node.is_a?(ClassStatement)
      end

      classes.map(&:to_hash)
    end

    # Returns an array of eneums parsed from the IO stream. The array
    # contains hashes with enum information and variables.
    #
    # The hash is the hash form of EnumStatement
    #
    # @example Accessing the enums
    #   parser.enums # => [{ name: 'test', variables: [], ...]
    #
    # @return [Array<Hash>] an array of hashes representing the class
    # @see EnumStatement
    def enums
      raise StreamNotParsed, 'you must parse first' if @tree.nil?

      enums = statements.select do |node|
        node.is_a?(EnumStatement)
      end

      enums.map(&:to_hash)
    end

    # Returns a hash representing the reason the parser failed.
    #
    # @example
    #   parser.last_error # => { reason: "Missing ';'", column: 1, line: 12 }
    def last_error
      {
        reason: @parser.failure_reason,
        column: @parser.failure_column,
        line: @parser.failure_line
      }
    end
    alias error last_error

    private

    # @api private
    def loaded?
      !@parser.nil?
    end

    # @api private
    def statements
      @tree.statements
    end

    # @api private
    def raise_parser_error!
      raise ParseError, "Parsing failed: #{error[:reason]} "\
                              "(col: #{error[:column]}, ln: #{error[:line]}"
    end

    # Fully reads the IO stream, and removes all comments
    # and obsolete tags
    def strip_comments_and_obsolete_tags!(io)
      io.read.strip.gsub(%r{\/\/.*}, '').gsub(/\;\s?obsolete.*/, ';')
    end
  end
end
